import cv2
from pymodaq.utils.daq_utils import ThreadCommand
from pymodaq.utils.data import DataFromPlugins, Axis
from pymodaq.control_modules.viewer_utility_classes import DAQ_Viewer_base, comon_parameters, main
from pymodaq.utils.parameter import Parameter

from pylablib.devices import Thorlabs
from qtpy import QtWidgets, QtCore
import numpy as np
from time import perf_counter


class DAQ_2DViewer_Thorlabs_TSI(DAQ_Viewer_base):
    """
    Plugin for Thorlabs scientific sCMOS cameras such as Kiralux or Zelux. It has been tested with Thorlabs Zelux camera on Windows.
    Building on pylablib driver, information about it can be found here : https://pylablib.readthedocs.io/en/stable/devices/Thorlabs_TLCamera.html

    As in pylablib, the plugin will look for DLLs in the default Thorcam installation folder. Specifying a custom DLL folder is not implemented yet.

    The plugin provides binning functionality as well as ROI (region of interest) selection, which are on handled the hardware side.
    To use ROIs, click on "Show/Hide ROI selection area" in the viewer panel (icon with dashed rectangle).
    Position the rectangle as you wish, either with mouse or by entering coordinates, then click "Update ROI" button.

    The "Clear ROI+Bin" button resets to default cameras parameters: no binning and full frame.
    """

    serialnumbers = Thorlabs.list_cameras_tlcam()

    params = comon_parameters + [
        {'title': 'Camera name:', 'name': 'camera_name', 'type': 'str', 'value': '', 'readonly': True},
        {'title': 'Serial number:', 'name': 'serial_number', 'type': 'list', 'limits': serialnumbers},
        #{'title': 'Sensor type:', 'name': 'sensor', 'type': 'str', 'value': '', 'readonly': True},
        #this will be used once pylablib accepts PR52
        {'title': 'Sensor type:', 'name': 'sensor', 'type': 'list', 'limits': ['Monochrome', 'Bayer']},
        {'title': 'Ouput Color:', 'name': 'output_color', 'type': 'list', 'limits': ['RGB', 'MonoChrome']},
        {'title': 'Update ROI', 'name': 'update_roi', 'type': 'bool_push', 'value': False},
        {'title': 'Clear ROI+Bin', 'name': 'clear_roi', 'type': 'bool_push', 'value': False},
        {'title': 'X binning', 'name': 'x_binning', 'type': 'int', 'value': 1},
        {'title': 'Y binning', 'name': 'y_binning', 'type': 'int', 'value': 1},
        {'title': 'Image width', 'name': 'hdet', 'type': 'int', 'value': 1, 'readonly': True},
        {'title': 'Image height', 'name': 'vdet', 'type': 'int', 'value': 1, 'readonly': True},
        {'title': 'Timing', 'name': 'timing_opts', 'type': 'group', 'children':
            [{'title': 'Exposure Time (ms)', 'name': 'exposure_time', 'type': 'int', 'value': 1},
            {'title': 'Compute FPS', 'name': 'fps_on', 'type': 'bool', 'value': True},
            {'title': 'FPS', 'name': 'fps', 'type': 'float', 'value': 0.0, 'readonly': True}]
        }
    ]

    callback_signal = QtCore.Signal()

    def ini_attributes(self):
        self.controller: Thorlabs.ThorlabsTLCamera = None

        self.x_axis = None
        self.y_axis = None
        self.last_tick = 0.0  # time counter used to compute FPS
        self.fps = 0.0

        self.data_shape: str = ''
        self.callback_thread = None

        # Disable "use ROI" option to avoid confusion with other buttons
        #self.settings.child('ROIselect', 'use_ROI').setOpts(visible=False)

    def commit_settings(self, param: Parameter):
        """Apply the consequences of a change of value in the detector settings

        Parameters
        ----------
        param: Parameter
            A given parameter (within detector_settings) whose value has been changed by the user
        """
        if param.name() == "exposure_time":
            self.controller.set_exposure(param.value()/1000)

        if param.name() == "fps_on":
            self.settings.child('timing_opts', 'fps').setOpts(visible=param.value())

        if param.name() == "update_roi":
            if param.value():   # Switching on ROI

                # We handle ROI and binning separately for clarity
                (old_x, _, old_y, _, xbin, ybin) = self.controller.get_roi() # Get current binning

                # Values need to be rescaled by binning factor and shifted by current x0,y0 to be correct.
                new_x = (old_x + self.settings.child('ROIselect', 'x0').value())*xbin
                new_y = (old_y + self.settings.child('ROIselect', 'y0').value())*xbin
                new_width = self.settings.child('ROIselect', 'width').value()*ybin
                new_height = self.settings.child('ROIselect', 'height').value()*ybin

                new_roi = (new_x, new_width, xbin, new_y, new_height, ybin)
                self.update_rois(new_roi)
                # recenter rectangle
                self.settings.child('ROIselect', 'x0').setValue(0)
                self.settings.child('ROIselect', 'y0').setValue(0)
                param.setValue(False)

        if param.name() in ['x_binning', 'y_binning']:
            # We handle ROI and binning separately for clarity
            (x0, w, y0, h, *_) = self.controller.get_roi()  # Get current ROI
            xbin = self.settings.child('x_binning').value()
            ybin = self.settings.child('y_binning').value()
            new_roi = (x0, w, xbin, y0, h, ybin)
            self.update_rois(new_roi)

        if param.name() == "clear_roi":
            if param.value():   # Switching on ROI
                wdet, hdet = self.controller.get_detector_size()
                # self.settings.child('ROIselect', 'x0').setValue(0)
                # self.settings.child('ROIselect', 'width').setValue(wdet)
                self.settings.child('x_binning').setValue(1)
                #
                # self.settings.child('ROIselect', 'y0').setValue(0)
                # new_height = self.settings.child('ROIselect', 'height').setValue(hdet)
                self.settings.child('y_binning').setValue(1)

                new_roi = (0, wdet, 1, 0, hdet, 1)
                self.update_rois(new_roi)
                param.setValue(False)

    def ini_detector(self, controller=None):
        """Detector communication initialization

        Parameters
        ----------
        controller: (object)
            custom object of a PyMoDAQ plugin (Slave case). None if only one actuator/detector by controller
            (Master case)

        Returns
        -------
        info: str
        initialized: bool
            False if initialization failed otherwise True
        """
        # Initialize camera class
        if not self.settings.child('serial_number').value() == '':
            self.ini_detector_init(old_controller=controller,
                                   new_controller=Thorlabs.ThorlabsTLCamera(self.settings.child('serial_number').value()))
        else:
            raise Exception('No compatible Thorlabs scientific camera was found.')

        device_info = self.controller.get_device_info()

        # Get camera name
        self.settings.child('camera_name').setValue(device_info.name)

        # this will be used once pylablib accepts PR52
        # # Get Sensor Type
        # self.settings.child('sensor').setValue(device_info.sensor_type)

        if 'monochrome' in self.settings['sensor'].lower():
            self.settings.child('output_color').setValue('MonoChrome')
            self.settings.child('output_color').setOpts(visible=False)

        # Set exposure time
        self.controller.set_exposure(self.settings.child('timing_opts', 'exposure_time').value()/1000)

        # FPS visibility
        self.settings.child('timing_opts', 'fps').setOpts(visible=self.settings.child('timing_opts', 'fps_on').value())

        # Update image parameters
        (*_, hbin, vbin) = self.controller.get_roi()
        height, width = self.controller.get_data_dimensions()
        self.settings.child('x_binning').setValue(hbin)
        self.settings.child('y_binning').setValue(vbin)
        self.settings.child('hdet').setValue(width)
        self.settings.child('vdet').setValue(height)

        # Way to define a wait function with arguments
        wait_func = lambda: self.controller.wait_for_frame(since='lastread', nframes=1, timeout=20.0)
        callback = ThorlabsCallback(wait_func)

        self.callback_thread = QtCore.QThread()  # creation of a Qt5 thread
        callback.moveToThread(self.callback_thread)  # callback object will live within this thread
        callback.data_sig.connect(
            self.emit_data)  # when the wait for acquisition returns (with data taken), emit_data will be fired

        self.callback_signal.connect(callback.wait_for_acquisition)
        self.callback_thread.callback = callback
        self.callback_thread.start()

        self._prepare_view()

        info = "Initialized camera"
        initialized = True
        return info, initialized

    def _prepare_view(self):
        """Preparing a data viewer by emitting temporary data. Typically, needs to be called whenever the
        ROIs are changed"""
        # wx = self.settings.child('rois', 'width').value()
        # wy = self.settings.child('rois', 'height').value()
        # bx = self.settings.child('rois', 'x_binning').value()
        # by = self.settings.child('rois', 'y_binning').value()
        #
        # sizex = wx // bx
        # sizey = wy // by
        height, width = self.controller.get_data_dimensions()

        self.settings.child('hdet').setValue(width)
        self.settings.child('vdet').setValue(height)
        mock_data = np.zeros((height, width))

        if width != 1 and height != 1:
            data_shape = 'Data2D'
        else:
            data_shape = 'Data1D'

        if data_shape != self.data_shape:
            self.data_shape = data_shape
            # init the viewers
            self.data_grabed_signal_temp.emit([DataFromPlugins(name='Thorlabs Camera',
                                                               data=[np.squeeze(mock_data)],
                                                               dim=self.data_shape,
                                                               labels=[f'ThorCam_{self.data_shape}'])])
            QtWidgets.QApplication.processEvents()

    def update_rois(self, new_roi):
        # In pylablib, ROIs compare as tuples
        (new_x, new_width, new_xbinning, new_y, new_height, new_ybinning) = new_roi
        if new_roi != self.controller.get_roi():
            # self.controller.set_attribute_value("ROIs",[new_roi])
            self.controller.set_roi(hstart=new_x, hend=new_x + new_width, vstart=new_y, vend=new_y + new_height,
                                    hbin=new_xbinning, vbin=new_ybinning)
            self.emit_status(ThreadCommand('Update_Status', [f'Changed ROI: {new_roi}']))
            self.controller.clear_acquisition()
            self.controller.setup_acquisition()
            # Finally, prepare view for displaying the new data
            self._prepare_view()

    def grab_data(self, Naverage=1, **kwargs):
        """
        Grabs the data. Synchronous method (kinda).
        ----------
        Naverage: (int) Number of averaging
        kwargs: (dict) of others optionals arguments
        """
        try:
            # Warning, acquisition_in_progress returns 1,0 and not a real bool
            if not self.controller.acquisition_in_progress():
                self.controller.clear_acquisition()
                self.controller.start_acquisition()
            #Then start the acquisition
            self.callback_signal.emit()  # will trigger the wait for acquisition

        except Exception as e:
            self.emit_status(ThreadCommand('Update_Status', [str(e), "log"]))

    def emit_data(self):
        """ Function used to emit data obtained by callback.

        Parameter
        ---------
        status: bool
            If True a frame is available, If False, a Timeout occured while waiting for the frame

        See Also
        --------
        daq_utils.ThreadCommand
        """
        try:
            # Get  data from buffer
            frame = self.controller.read_newest_image()
            # Emit the frame.
            if frame is not None:       # happens for last frame when stopping camera
                if self.settings['output_color'] == 'RGB':
                    rgb_image = cv2.cvtColor(frame, cv2.COLOR_BAYER_BG2RGB)
                    self.data_grabed_signal.emit([DataFromPlugins(name='Thorlabs Camera',
                                                                  data=[np.squeeze(rgb_image[..., ind]) for ind in
                                                                        range(3)],
                                                                  dim=self.data_shape,
                                                                  labels=[f'ThorCam_{self.data_shape}'])])
                else:
                    if 'monochrome' in self.settings['sensor'].lower():
                        self.data_grabed_signal.emit([DataFromPlugins(name='Thorlabs Camera',
                                                                      data=[np.squeeze(frame)],
                                                                      dim=self.data_shape,
                                                                      labels=[f'ThorCam_{self.data_shape}'])])
                    else:
                        grey_image = cv2.cvtColor(frame, cv2.COLOR_BAYER_BG2GRAY)
                        self.data_grabed_signal.emit([DataFromPlugins(name='Thorlabs Camera',
                                                                      data=[np.squeeze(grey_image)],
                                                                      dim=self.data_shape,
                                                                      labels=[f'ThorCam_{self.data_shape}'])])

            if self.settings.child('timing_opts', 'fps_on').value():
                self.update_fps()

            # To make sure that timed events are executed in continuous grab mode
            QtWidgets.QApplication.processEvents()

        except Exception as e:
            self.emit_status(ThreadCommand('Update_Status', [str(e), 'log']))

    def update_fps(self):
        current_tick = perf_counter()
        frame_time = current_tick-self.last_tick

        if self.last_tick != 0.0 and frame_time != 0.0:
            # We don't update FPS for the first frame, and we also avoid divisions by zero

            if self.fps == 0.0:
                self.fps = 1 / frame_time
            else:
                # If we already have an FPS calculated, we smooth its evolution
                self.fps = 0.9 * self.fps + 0.1 / frame_time

        self.last_tick = current_tick

        # Update reading
        self.settings.child('timing_opts', 'fps').setValue(round(self.fps, 1))

    def close(self):
        """
        Terminate the communication protocol
        """
        # Terminate the communication
        self.controller.close()
        self.controller = None  # Garbage collect the controller
        self.status.initialized = False
        self.status.controller = None
        self.status.info = ""

    def stop(self):
        """Stop the acquisition."""
        self.controller.clear_acquisition()
        return ''


class ThorlabsCallback(QtCore.QObject):
    """Callback object """
    data_sig = QtCore.Signal()

    def __init__(self, wait_fn):
        super().__init__()
        # Set the wait function
        self.wait_fn = wait_fn

    def wait_for_acquisition(self):
        try:
            new_data = self.wait_fn()
            if new_data is not False:  # will be returned if the main thread called CancelWait
                self.data_sig.emit()
        except Thorlabs.ThorlabsTimeoutError:
            pass


if __name__ == '__main__':
    main(__file__, init=False)



# -*- coding: utf-8 -*-
"""
Created the 31/08/2023

@author: Sebastien Weber
"""
from pathlib import Path

from pymodaq.utils.config import BaseConfig, USER


class Config(BaseConfig):
    """Main class to deal with configuration values for this plugin"""
    config_template_path = Path(__file__).parent.joinpath('resources/config_template.toml')
    config_name = f"config_{__package__.split('pymodaq_plugins_')[1]}"



import clr
import sys

from System import Decimal
from System import Action
from System import UInt64
from System import UInt32

kinesis_path = 'C:\\Program Files\\Thorlabs\\Kinesis'
sys.path.append(kinesis_path)

clr.AddReference("Thorlabs.MotionControl.DeviceManagerCLI")
clr.AddReference("Thorlabs.MotionControl.IntegratedStepperMotorsCLI")
clr.AddReference("Thorlabs.MotionControl.GenericMotorCLI")
clr.AddReference("Thorlabs.MotionControl.IntegratedStepperMotorsCLI")
clr.AddReference("Thorlabs.MotionControl.DeviceManagerCLI")
clr.AddReference("Thorlabs.MotionControl.GenericMotorCLI")
clr.AddReference("Thorlabs.MotionControl.FilterFlipperCLI")

import Thorlabs.MotionControl.FilterFlipperCLI as FilterFlipper
import Thorlabs.MotionControl.IntegratedStepperMotorsCLI as Integrated
import Thorlabs.MotionControl.DeviceManagerCLI as Device
import Thorlabs.MotionControl.GenericMotorCLI as Generic


Device.DeviceManagerCLI.BuildDeviceList()
serialnumbers_integrated_stepper = [str(ser) for ser in Device.DeviceManagerCLI.GetDeviceList(Integrated.CageRotator.DevicePrefix)]
serialnumbers_flipper = [str(ser) for ser in Device.DeviceManagerCLI.GetDeviceList(FilterFlipper.FilterFlipper.DevicePrefix)]


class Kinesis:

    def __init__(self):
        self._device = None

    def connect(self, serial: int):
        self._device.Connect(serial)
        self._device.WaitForSettingsInitialized(5000)
        self._device.StartPolling(250)

    def close(self):
        """
            close the current instance of Kinesis instrument.
        """
        self._device.StopPolling()
        self._device.Disconnect()
        self._device.Dispose()
        self._device = None

    @property
    def name(self) -> str:
        return self._device.GetDeviceInfo().Name

    @property
    def backlash(self):
        return Decimal.ToDouble(self._device.GetBacklash())

    @backlash.setter
    def backlash(self, backlash: float):
        self._device.SetBacklash(Decimal(backlash))

    def stop(self):
        self._device.Stop(0)

    def move_abs(self, position: float, callback=None):
        if callback is not None:
            callback = Action[UInt64](callback)
        else:
            callback = 0
        self._device.MoveTo(Decimal(position), callback)

    def move_rel(self, position: float, callback=None):
        if callback is not None:
            callback = Action[UInt64](callback)
        else:
            callback = 0
        self._device.MoveRelative(Generic.MotorDirection.Forward, Decimal(position), callback)

    def home(self, callback=None):
        if callback is not None:
            callback = Action[UInt64](callback)
        else:
            callback = 0
        self._device.Home(callback)

    def get_position(self):
        raise NotImplementedError


class IntegratedStepper(Kinesis):

    def __init__(self):
        self._device: Integrated.CageRotator = None

    def connect(self, serial: int):
        if serial in serialnumbers_integrated_stepper:
            self._device = Integrated.CageRotator.CreateCageRotator(serial)
            super().connect(serial)
            if not (self._device.IsSettingsInitialized()):
                raise (Exception("no Stage Connected"))
            else:
                self._device.LoadMotorConfiguration(serial)
        else:
            raise ValueError('Invalid Serial Number')

    def get_position(self):
        return Decimal.ToDouble(self._device.ContinuousRotationPosition)


class Flipper(Kinesis):
    def __init__(self):
        self._device: FilterFlipper.FilterFlipper = None

    def connect(self, serial: int):
        if serial in serialnumbers_flipper:
            self._device = FilterFlipper.FilterFlipper.CreateFilterFlipper(serial)
            super().connect(serial)
            if not (self._device.IsSettingsInitialized()):
                raise (Exception("no Stage Connected"))
        else:
            raise ValueError('Invalid Serial Number')

    def move_abs(self, position: float, callback=None):
        if int(position) == 1:
            position = 1
        else:
            position = 2
        self._device.SetPosition(UInt32(position), 0)

    def get_position(self):
        position = int(self._device.Position)
        if position == 2:
            position = 0
        else:
            position = 1
        return position



"""
Plugin for the K10CR1 Integrated Stepper motor for rotation
Its is using the Instrumental package with its kinesis driver (based from the C-libraries)
Thorlabs Kinesis drivers and SDKs should be installed and the path to where are located the dlls should be added to
the PATH system environment variable
"""

from pymodaq.control_modules.move_utility_classes import DAQ_Move_base, main
from pymodaq.control_modules.move_utility_classes import comon_parameters
from pymodaq.utils.daq_utils import ThreadCommand, getLineInfo, find_dict_in_list_from_key_val
from easydict import EasyDict as edict
from instrumental.drivers.motion.kinesis import list_instruments
from instrumental import instrument

parameter_sets = list_instruments()
serialnumbers = [pset['serial'] for pset in parameter_sets if pset['classname'] == 'K10CR1']

class DAQ_Move_K10CR1(DAQ_Move_base):
    """
        Wrapper object to access the kinesis fonctionnalities, similar wrapper for all controllers.

        =============== ==================
        **Attributes**   **Type**
        *kinesis_path*   string
        *serialnumbers*  int list
        *params*         dictionnary list
        =============== ==================

        See Also
        --------
        daq_utils.ThreadCommand

    """
    _controller_units = 'degrees'
    is_multiaxes = False

    stage_names = []

    params = [{'title': 'Controller ID:', 'name': 'controller_id', 'type': 'str', 'value': '', 'readonly': True},
              {'title': 'Serial number:', 'name': 'serial_number', 'type': 'list', 'limits': serialnumbers},
              {'title': 'MultiAxes:', 'name': 'multiaxes', 'type': 'group', 'visible': is_multiaxes, 'children': [
                  {'title': 'is Multiaxes:', 'name': 'ismultiaxes', 'type': 'bool', 'value': is_multiaxes,
                   'default': False},
                  {'title': 'Status:', 'name': 'multi_status', 'type': 'list', 'value': 'Master',
                   'limits': ['Master', 'Slave']},
                  {'title': 'Axis:', 'name': 'axis', 'type': 'list', 'limits': stage_names},

              ]}] + comon_parameters

    def __init__(self, parent=None, params_state=None):
        super().__init__(parent, params_state)

        self.controller = None
        self.settings.child('epsilon').setValue(0.005)


    def commit_settings(self, param):
        """
            | Activate any parameter changes on the hardware.
            | Called after a param_tree_changed signal received from DAQ_Move_main.

            =============== ================================ ========================
            **Parameters**  **Type**                          **Description**
            *param*         instance of pyqtgraph parameter  The parameter to update
            =============== ================================ ========================
        """
        pass

    def ini_stage(self, controller=None):
        """
            Initialize the controller and stages (axes) with given parameters.

            =============== ================================================ =========================================================================================
            **Parameters**   **Type**                                         **Description**
            *controller*     instance of the specific controller object       If defined this hardware will use it and will not initialize its own controller instance
            *stage*          instance of the stage (axis or whatever) object  ???
            =============== ================================================ =========================================================================================

            Returns
            -------
            Easydict
                dictionnary containing keys:
                 * *info* : string displaying various info
                 * *controller*: instance of the controller object in order to control other axes without the need to init the same controller twice
                 * *stage*: instance of the stage (axis or whatever) object
                 * *initialized*: boolean indicating if initialization has been done corretly

            See Also
            --------
            daq_utils.ThreadCommand
        """
        try:
            self.status.update(edict(info="", controller=None, initialized=False))

            # check whether this stage is controlled by a multiaxe controller (to be defined for each plugin)

            # if mutliaxes then init the controller here if Master state otherwise use external controller
            if self.settings.child('multiaxes', 'ismultiaxes').value() and self.settings.child('multiaxes',
                                                                                               'multi_status').value() == "Slave":
                if controller is None:
                    raise Exception('no controller has been defined externally while this axe is a slave one')
                else:
                    self.controller = controller
            else:  # Master stage

                self.controller = instrument(
                    find_dict_in_list_from_key_val(parameter_sets, 'serial',
                                                   self.settings.child('serial_number').value()))
                self.controller.backlash = '0deg'
            info = self.controller.get_info()
            self.settings.child('controller_id').setValue(info)

            self.status.info = info
            self.status.controller = self.controller
            self.status.initialized = True
            return self.status

        except Exception as e:
            self.emit_status(ThreadCommand('Update_Status', [getLineInfo() + str(e), 'log']))
            self.status.info = getLineInfo() + str(e)
            self.status.initialized = False
            return self.status

    def close(self):
        """
            close the current instance of Kinesis instrument.
        """
        self.controller.close()
        self.controller = None

    def stop_motion(self):
        """
            See Also
            --------
            DAQ_Move_base.move_done
        """
        self.controller.stop()
        self.move_done()

    def check_position(self):
        """
            Get the current hardware position with scaling conversion of the Kinsesis insrument provided by get_position_with_scaling

            See Also
            --------
            DAQ_Move_base.get_position_with_scaling, daq_utils.ThreadCommand
        """
        pos = self.controller.position.m_as('deg')
        pos = self.get_position_with_scaling(pos)
        self.emit_status(ThreadCommand('check_position', [pos]))
        return pos

    def move_Abs(self, position):
        """
            Make the hardware absolute move from the given position of the Kinesis instrument after thread command signal was received in DAQ_Move_main.

            =============== ========= =======================
            **Parameters**  **Type**   **Description**

            *position*       float     The absolute position
            =============== ========= =======================

            See Also
            --------
            DAQ_Move_base.set_position_with_scaling

        """
        position = self.check_bound(position)
        self.target_position = position
        position = self.set_position_with_scaling(position)

        self.controller.move_to(f'{position}deg', wait=False)
        #self.poll_moving()

    def move_Rel(self, position):
        """

        """
        position = self.check_bound(self.current_position + position) - self.current_position
        self.target_position = position + self.current_position
        position = self.set_position_relative_with_scaling(position)
        self.controller.move_relative(f'{position}deg', wait=False)
        #self.poll_moving()

    def move_Home(self):
        """
            Make the absolute move to original position (0).
        """
        self.target_position = 0.
        self.controller.home()
        #self.poll_moving()


if __name__ == '__main__':
    main(__file__, init=False)


from pymodaq.control_modules.move_utility_classes import DAQ_Move_base, main
from pymodaq.control_modules.move_utility_classes import comon_parameters_fun
from pymodaq.utils.daq_utils import ThreadCommand, getLineInfo
from pymodaq.utils.logger import set_logger, get_module_name
import clr
from qtpy import QtWidgets
import sys
from easydict import EasyDict as edict

from System import Decimal
from System import Action
from System import UInt64

kinesis_path = 'C:\\Program Files\\Thorlabs\\Kinesis'
sys.path.append(kinesis_path)
clr.AddReference("Thorlabs.MotionControl.DeviceManagerCLI")
clr.AddReference("Thorlabs.MotionControl.IntegratedStepperMotorsCLI")
clr.AddReference("Thorlabs.MotionControl.GenericMotorCLI")
import Thorlabs.MotionControl.IntegratedStepperMotorsCLI as Integrated
import Thorlabs.MotionControl.DeviceManagerCLI as Device
import Thorlabs.MotionControl.GenericMotorCLI as Generic

MOVEDONEACTION = False
logger = set_logger(get_module_name(__file__))


class DAQ_Move_Kinesis(DAQ_Move_base):
    """
        Wrapper object to access the kinesis fonctionnalities, similar wrapper for all controllers.

        =============== ==================
        **Attributes**   **Type**
        *kinesis_path*   string
        *serialnumbers*  int list
        *params*         dictionnary list
        =============== ==================

        See Also
        --------
        daq_utils.ThreadCommand

    """
    _controller_units = 'degrees'
    _epsilon = 0.05

    try:

        Device.DeviceManagerCLI.BuildDeviceList()
        serialnumbers = [str(ser) for ser in Device.DeviceManagerCLI.GetDeviceList(Integrated.CageRotator.DevicePrefix)]

    except:
        serialnumbers = []
    is_multiaxes = False

    stage_names = []

    params = [{'title': 'Kinesis library:', 'name': 'kinesis_lib', 'type': 'browsepath', 'value': kinesis_path},
              {'title': 'Controller ID:', 'name': 'controller_id', 'type': 'str', 'value': '', 'readonly': True},
              {'title': 'Serial number:', 'name': 'serial_number', 'type': 'list', 'limits': serialnumbers},
              ] + comon_parameters_fun(is_multiaxes, epsilon=_epsilon)


    def ini_attributes(self):

        self.controller: Integrated.CageRotator.CreateCageRotator = None

        try:
            if MOVEDONEACTION:
                self.move_done_action = Action[UInt64](self.someaction)  # if selected wil trigger
                # self.someaction when move is done

            else:
                self.move_done_action = 0  # if selected will return immediately


        except Exception as e:
            self.emit_status(ThreadCommand("Update_Status", [getLineInfo() + str(e), 'log']))
            raise Exception(getLineInfo() + str(e))

    def someaction(self, pos_action):
        position = self.get_actuator_value()
        logger.debug(f'posaction is {pos_action} and position is {position}')

    def commit_settings(self, param):
        """
            | Activate any parameter changes on the hardware.
            | Called after a param_tree_changed signal received from DAQ_Move_main.

            =============== ================================ ========================
            **Parameters**  **Type**                          **Description**
            *param*         instance of pyqtgraph parameter  The parameter to update
            =============== ================================ ========================
        """
        if param.name() == 'kinesis_lib':
            try:
                Device.DeviceManagerCLI.BuildDeviceList()
                serialnumbers = [str(ser) for ser in
                                 Device.DeviceManagerCLI.GetDeviceList(Integrated.CageRotator.DevicePrefix)]

            except:
                serialnumbers = []
            self.settings.child('serial_number').setOpts(limits=serialnumbers)

    def ini_stage(self, controller=None):
        """
        """

        Device.DeviceManagerCLI.BuildDeviceList()
        serialnumbers = Device.DeviceManagerCLI.GetDeviceList(Integrated.CageRotator.DevicePrefix)
        ser_bool = self.settings['serial_number'] in serialnumbers
        if ser_bool:
            self.controller = self.ini_stage_init(controller,
                                                  Integrated.CageRotator.CreateCageRotator(
                                                      self.settings['serial_number']))
        else:
            raise Exception("Not valid serial number")

        if self.settings['multiaxes', 'multi_status'] == "Master":

            self.controller.Connect(self.settings['serial_number'])
            self.controller.WaitForSettingsInitialized(5000)
            self.controller.StartPolling(250)

        info = self.controller.GetDeviceInfo().Name
        self.settings.child('controller_id').setValue(info)
        if not (self.controller.IsSettingsInitialized()):
            raise (Exception("no Stage Connected"))
        else:
            self.controller.LoadMotorConfiguration(self.settings['serial_number'])
        self.controller.SetBacklash(Decimal(0))

        initialized = True
        return info, initialized

    def close(self):
        """
            close the current instance of Kinesis instrument.
        """
        self.controller.StopPolling()
        self.controller.Disconnect()
        self.controller.Dispose()
        self.controller = None

    def stop_motion(self):
        """
            See Also
            --------
            DAQ_Move_base.move_done
        """
        self.controller.stop(0)
        self.move_done()

    def get_actuator_value(self):
        """
            Get the current hardware position with scaling conversion of the Kinsesis insrument provided by get_position_with_scaling

            See Also
            --------
            DAQ_Move_base.get_position_with_scaling, daq_utils.ThreadCommand
        """

        pos = Decimal.ToDouble(self.controller.ContinuousRotationPosition)
        logger.debug(f'Motor state is {self.controller.State} and position is {pos}')
        pos = self.get_position_with_scaling(pos)
        self.emit_status(ThreadCommand('check_position', [pos]))
        return pos

    def move_abs(self, position):
        """
            Make the hardware absolute move from the given position of the Kinesis instrument after thread command signal was received in DAQ_Move_main.

            =============== ========= =======================
            **Parameters**  **Type**   **Description**

            *position*       float     The absolute position
            =============== ========= =======================

            See Also
            --------
            DAQ_Move_base.set_position_with_scaling

        """
        position = self.check_bound(position)
        self.target_position = position

        position = self.set_position_with_scaling(position)
        logger.debug(f'Should move to {position} but decimal gets {Decimal.ToDouble(Decimal(position))}')
        self.controller.MoveTo(Decimal(position), self.move_done_action)
        QtWidgets.QApplication.processEvents()

    def move_rel(self, position):
        """
            | Make the hardware relative move from the given position of the Kinesis instrument after thread command signal was received in DAQ_Move_main.
            |
            | The final target position is given by **current_position+position**.

            =============== ========= =======================
            **Parameters**  **Type**   **Description**

            *position*       float     The absolute position
            =============== ========= =======================

            See Also
            --------
            DAQ_Move_base.set_position_with_scaling

        """
        position = self.check_bound(self.current_position + position) - self.current_position
        self.target_position = position + self.current_position
        logger.debug(f'Should move to {Decimal.ToDouble(Decimal(self.target_position))}')
        position = self.set_position_relative_with_scaling(position)

        self.controller.MoveRelative(Generic.MotorDirection.Forward, Decimal(position), self.move_done_action)

    def move_home(self):
        """
            Make the absolute move to original position (0).
        """
        self.controller.Home(self.move_done_action)


if __name__ == '__main__':
    main(__file__, init=False)



"""
Description:
------------

python wrapper around the TLPM.dll from Thorlabs
it supports the following Thorlabs power meters and appropriate sensors and can administer them in parallel.
· PM101x (PM101, PM101A, PM101R, PM101U)
· PM102x (PM102, PM102A, PM102U)
· PM103x (PM103, PM103A, PM103U)
· PM100USB
· PM16-Series
· PM160 1)
· PM160T 1)
· PM160T-HP 1)
· PM400
· PM100A
· PM100D
· PM200

Installation:
-------------

* Download the Optical Power Monitor software from thorlabs website:
  https://www.thorlabs.com/software_pages/ViewSoftwarePage.cfm?Code=OPM
* after installation, the dll driver and TLPM.py wrapper will be installed in the
  C:\ProgramFiles\IVI Foundation\VISA\Win64\ or  C:\ProgramFiles (x86)\IVI Foundation\VISA\WinNT\
  under the environment variable VXIPNPPATH or VXIPNPPATH64
  >>>import os
  >>>os.environ['VXIPNPPATH']
"""

import os
import sys
import importlib
from pathlib import Path
import ctypes
import functools

from pymodaq.utils import daq_utils as utils
from pymodaq.utils.logger import set_logger, get_module_name
logger = set_logger(get_module_name(__file__))
if utils.is_64bits():
    path_dll = str(Path(os.environ['VXIPNPPATH64']).joinpath('Win64', 'Bin'))
else:
    path_dll = str(Path(os.environ['VXIPNPPATH']).joinpath('WinNT', 'Bin'))
os.add_dll_directory(path_dll)

try:
    path_python_wrapper = Path(os.environ['VXIPNPPATH']).joinpath('WinNT', 'TLPM', 'Example', 'Python')
    sys.path.insert(0, str(path_python_wrapper))
    import TLPM
except ModuleNotFoundError as e:
    error = f"The *TLPM.py* python wrapper of thorlabs TLPM dll could not be located on your system. Check if present" \
            f" in {path_dll}"
    raise ModuleNotFoundError(error)


def error_handling(default_arg=None):
    """decorator around TLPM functions to handle return if errors"""
    def error_management(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            try:
                ret = func(*args, **kwargs)
                return ret
            except Exception as e:
                logger.exception(f'The function {func.__name__} returned the error: {e}')
                return default_arg
        return wrapper
    return error_management


class DeviceInfo:
    def __init__(self, model_name='', serial_number='', manufacturer='', is_available=False):
        self.model_name = model_name
        self.serial_number = serial_number
        self.manufacturer = manufacturer
        self.is_available = is_available

    def __repr__(self):
        return f'Model: {self.model_name} / SN: {self.serial_number} by {self.manufacturer} is'\
               f' {"" if self.is_available else "not"} available'


class GetInfos:
    def __init__(self, tlpm=None):
        if tlpm is None:
            tlpm = TLPM.TLPM()
        self._tlpm = tlpm
        self._Ndevices = 0

    @error_handling(0)
    def get_connected_ressources_number(self):
        deviceCount = ctypes.c_uint32()
        self._tlpm.findRsrc(ctypes.byref(deviceCount))
        self._Ndevices = deviceCount.value
        return self._Ndevices

    def get_devices_name(self):
        self.get_connected_ressources_number()
        names = []
        resource_name = ctypes.create_string_buffer(1024)
        for ind in range(self._Ndevices):
            self._tlpm.getRsrcName(ctypes.c_int(ind), resource_name)
            names.append(resource_name.value.decode())
        return names

    @error_handling(DeviceInfo())
    def get_devices_info(self, index: int):
        self.get_connected_ressources_number()
        if index >= self._Ndevices:
            return DeviceInfo()
        modelName = ctypes.create_string_buffer(1024)
        serialNumber = ctypes.create_string_buffer(1024)
        manufacturer = ctypes.create_string_buffer(1024)
        is_available = ctypes.c_int16()
        self._tlpm.getRsrcInfo(index, modelName, serialNumber, manufacturer, ctypes.byref(is_available))
        return DeviceInfo(modelName.value.decode(), serialNumber.value.decode(),
                          manufacturer.value.decode(), bool(is_available.value))


infos = GetInfos()
Ndevices = infos.get_connected_ressources_number()
DEVICE_NAMES = infos.get_devices_name()


class CustomTLPM:
    def __init__(self, index=None):
        super().__init__()
        self._index = index
        self._tlpm = TLPM.TLPM()
        self.infos = GetInfos(self._tlpm)

    def __enter__(self):
        device_name = self.infos.get_devices_name()[self._index]
        self.open(device_name)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    def open_by_index(self, index=None):
        if index is not None:
            self._index = index
        device_name = self.infos.get_devices_name()[self._index]
        self.open(device_name)

    @error_handling(False)
    def open(self, resource_name: str, id_query=True, reset=True):
        resource = ctypes.create_string_buffer(1024)
        resource.value = resource_name.encode()
        id_query = ctypes.c_bool(id_query)
        reset = ctypes.c_bool(reset)
        self._tlpm.open(resource, id_query, reset)
        return True

    @error_handling()
    def close(self):
        self._tlpm.close()

    @error_handling('')
    def get_calibration(self):
        message = ctypes.create_string_buffer(1024)
        self._tlpm.getCalibrationMsg(message)
        return message.value.decode()

    @error_handling(0.)
    def get_power(self):
        power = ctypes.c_double()
        self._tlpm.measPower(ctypes.byref(power))
        return power.value

    @property
    @error_handling((500, 800))
    def wavelength_range(self):
        wavelength_min = ctypes.c_double()
        wavelength_max = ctypes.c_double()

        self._tlpm.getWavelength(TLPM.TLPM_ATTR_MIN_VAL, ctypes.byref(wavelength_min))
        self._tlpm.getWavelength(TLPM.TLPM_ATTR_MAX_VAL, ctypes.byref(wavelength_max))
        return wavelength_min.value, wavelength_max.value

    @property
    @error_handling(-1)
    def wavelength(self):
        wavelength = ctypes.c_double()
        self._tlpm.getWavelength(TLPM.TLPM_ATTR_SET_VAL, ctypes.byref(wavelength))
        return wavelength.value

    @wavelength.setter
    @error_handling()
    def wavelength(self, wavelength: float):
        wavelength = ctypes.c_double(wavelength)
        self._tlpm.setWavelength(wavelength)


if __name__ == '__main__':
    from time import sleep
    print(Ndevices)
    print(DEVICE_NAMES)

    with CustomTLPM(0) as tlpm:
        print(tlpm.wavelength)
        tlpm.wavelength = 532.
        sleep(1)
        print(tlpm.wavelength)
        print(tlpm.get_power())





from pymodaq.control_modules.move_utility_classes import DAQ_Move_base, main
from pymodaq.control_modules.move_utility_classes import comon_parameters_fun
from pymodaq.utils.daq_utils import ThreadCommand, getLineInfo
from easydict import EasyDict as edict

import pylablib.devices.Thorlabs as Thorlabs



class DAQ_Move_PRM1Z8_pylablib(DAQ_Move_base):
    """
        Wrapper object to access the Flipper functionalities, similar wrapper for all controllers.
        =============== ==============
        **Attributes**    **Type**
        *params*          dictionnary
        =============== ==============
    """
    _controller_units = 'deg'
    is_multiaxes = False
    stage_names = []
    _epsilon = 0.1
    _dvc = Thorlabs.list_kinesis_devices()
    #serialnumbers = [d[0] for d in _dvc if d[1] == 'APT DC Motor Controller']
    serialnumbers = [d[0] for d in _dvc]

    params= [{'title': 'Controller ID:', 'name': 'controller_id', 'type': 'str', 'value': '', 'readonly': True},
             {'title': 'Serial number:', 'name': 'serial_number', 'type': 'list', 'limits': serialnumbers},
             {'title': 'Home Position:', 'name': 'home_position', 'type': 'float', 'value': 0.0},
             {'title': 'Set Zero', 'name': 'set_zero', 'type': 'bool_push', 'value': False},
             {'title': 'Reset Home', 'name': 'reset_home', 'type': 'bool_push', 'value': False},

             ] + comon_parameters_fun(is_multiaxes, stage_names, epsilon=_epsilon)

    def __init__(self, parent=None, params_state=None):
        super().__init__(parent, params_state)
        self.settings.child('epsilon').setValue(0.005)
        self.settings.child('epsilon').setReadonly()

        self.settings.child('timeout').setValue(100)

        #Scaling and bounds can be set to False because they are just not needed
        self.settings.child('bounds', 'is_bounds').setValue(False)
        self.settings.child('bounds', 'is_bounds').setReadonly()
        self.settings.child('bounds').hide()
        self.settings.child('scaling', 'use_scaling').setValue(False)
        self.settings.child('scaling', 'use_scaling').setReadonly()
        self.settings.child('scaling').hide()

        #Home is fixed (for now)
        # self.settings.child('home_position').setReadonly()

        # self.settings.child('scaling','scaling').setReadonly()
        # self.settings.child('scaling','offset').setReadonly()

    def commit_settings(self,param):
        """
            | Activate any parameter changes on the hardware.
            | Called after a param_tree_changed signal received from DAQ_Move_main.

            =============== ================================ ========================
            **Parameters**  **Type**                          **Description**
            *param*         instance of pyqtgraph Parameter  The parameter to update
            =============== ================================ ========================
        """
        if param.name() == 'set_zero':
            if param.value() == True:
                self.controller.set_position_reference(scale=True)
                self.check_position()
                self.settings.child('set_zero').setValue(False)
        elif param.name() == 'reset_home':
            if param.value() == True:
                self.controller.home(force=True, timeout=self.settings.child('timeout').value())
                self.check_position()
                self.settings.child('reset_home').setValue(False)

    def ini_stage(self,controller=None):
        """Initialize the controller and stages (axes) with given parameters.

            ============== =========================================== ===========================================================================================
            **Parameters**  **Type**                                     **Description**

            *controller*    instance of the specific controller object  If defined this hardware will use it and will not initialize its own controller instance
            ============== =========================================== ===========================================================================================

            Returns
            -------
            Easydict
                dictionnary containing keys:
                 * *info* : string displaying various info
                 * *controller*: instance of the controller object in order to control other axes without the need to init the same controller twice
                 * *stage*: instance of the stage (axis or whatever) object
                 * *initialized*: boolean indicating if initialization has been done corretly

            See Also
            --------
            daq_utils.ThreadCommand
        """
        try:
            self.status.update(edict(info="",controller=None,initialized=False))

            #check whether this stage is controlled by a multiaxe controller (to be defined for each plugin)

            # if multiaxes then init the controller here if Master state otherwise use external controller
            if self.settings.child('multiaxes','ismultiaxes').value() and self.settings.child('multiaxes','multi_status').value()=="Slave":
                if controller is None: 
                    raise Exception('no controller has been defined externally while this axe is a slave one')
                else:
                    self.controller = controller
            else: #Master stage
                try:
                    controller = Thorlabs.kinesis.KinesisMotor(self.settings.child('serial_number').value(),scale='stage')
                except:
                    raise Exception(f'No controller found with serial number {self.settings.child("serial_number").value()}')
                else:
                    self.controller = controller

            if not self.controller.is_opened():
                self.controller.open()

            #Getting the information from the device
            info = self.controller.get_device_info()

            #Setting the name of the controller
            #asserting that the controller is scale_aware and
            stage_name = self.controller.get_stage()
            assert stage_name == 'PRM1-Z8'
            self.settings.child('controller_id').setValue(stage_name)

            unit = self.controller.get_scale_units()
            assert unit == 'deg'

            #IDK what that does but ok
            self.status.info = info.notes
            self.status.controller = self.controller
            self.status.initialized = True
            return self.status
        #This is in case something fails
        except Exception as e:
            self.emit_status(ThreadCommand('Update_Status',[getLineInfo()+ str(e),'log']))
            self.status.info=getLineInfo()+ str(e)
            self.status.initialized=False
            return self.status

    def close(self):
        """
            close the current instance of Kinesis Flipper instrument.
        """
        self.controller.close()
        self.controller = None

    def stop_motion(self):
        """
            See Also
            --------
            DAQ_Move_base.move_done
        """
        # self.controller.stop(0)
        self.move_done()

    def check_position(self):
        """
            Get the current hardware position with scaling conversion of the Kinesis
            instrument provided by get_position_with_scaling

            See Also
            --------
            DAQ_Move_base.get_position_with_scaling, daq_utils.ThreadCommand
        """
        #Get position = 0 or 1, possibly None if unknown
        pos = self.controller.get_position()
        #Repoll if pos returns none
        while pos == None:
            pos = self.controller.get_position()

        #This is superfluous as there is not scaling
        # pos = self.get_position_with_scaling(pos)
        self.emit_status(ThreadCommand('check_position', [pos]))
        return pos

    def move_abs(self,position):
        """
            Make the hardware absolute move from the given position after thread command signal was received in DAQ_Move_main.

            =============== ========= =======================
            **Parameters**  **Type**   **Description**

            *position*       int       either 1 or 2 for the flipper
            =============== ========= =======================

            See Also
            --------
            DAQ_Move_base.set_position_with_scaling

        """
        position = self.check_bound(position)

        self.controller.move_to(position)
        self.emit_status(ThreadCommand('Update_Status', [f'Moving to position: {position}']))

        self.target_position = position
        self.poll_moving()

    def move_rel(self,position):
        """ Move the actuator to the relative target actuator value defined by position
        Parameters
        ----------
        position: (float) value of the relative target positioning
        """
        self.current_position = self.check_position()
        # rel_move  = self.check_bound(self.current_position+position)-self.current_position
        new_pos = self.check_bound(self.current_position + position)
        self.target_position = new_pos

        # self.target_position = position
        self.controller.move_to(new_pos)
        self.emit_status(ThreadCommand('Update_Status', [f'Moving to position: {position}']))
        self.poll_moving()

    def move_home(self):
        """
            Make the absolute move to original position (0).
        """
        home = self.settings.child('home_position').value()
        self.target_position = home
        self.controller.move_to(home)
        self.emit_status(ThreadCommand('Update_Status', [f'Moving home: {home}']))
        self.poll_moving()


if __name__ == '__main__':
    main(__file__, init=False)


# -*- coding: utf-8 -*-
"""
Created the 15/06/2023

@author: Sebastien Weber
"""

from pymodaq.control_modules.move_utility_classes import DAQ_Move_base, comon_parameters_fun, main  # common set of parameters for all actuators
from pymodaq.utils.daq_utils import ThreadCommand # object used to send info back to the main thread
from pymodaq.utils.parameter import Parameter

from elliptec import Controller, Rotator
from elliptec.scan import find_ports, scan_for_devices

com_ports = find_ports()


class DAQ_Move_Elliptec(DAQ_Move_base):
    """Plugin for the Template Instrument

    This object inherits all functionality to communicate with PyMoDAQ Module through inheritance via DAQ_Move_base
    It then implements the particular communication with the instrument

    Attributes:
    -----------
    controller: object
        The particular object that allow the communication with the hardware, in general a python wrapper around the
         hardware library

    """
    _controller_units = 'whatever'
    is_multiaxes = True
    axes_names = ['0']
    _epsilon = 0.1

    params = [ {'title': 'COM port', 'name': 'com_port', 'type': 'list', 'limits': com_ports},
               {'title': 'Serial No.', 'name': 'serial', 'type': 'str'},
               {'title': 'Motor Type', 'name': 'motor', 'type': 'str'},
               {'title': 'Range', 'name': 'range', 'type': 'str'},
               ] + comon_parameters_fun(is_multiaxes, axes_names, epsilon=_epsilon)

    def ini_attributes(self):
        self.controller: Rotator = None

    def get_actuator_value(self):
        """Get the current value from the hardware with scaling conversion.

        Returns
        -------
        float: The position obtained after scaling conversion.
        """
        pos = self.controller.get_angle()
        pos = self.get_position_with_scaling(pos)
        return pos

    def close(self):
        """Terminate the communication protocol"""
        self.controller.close()

    def commit_settings(self, param: Parameter):
        """Apply the consequences of a change of value in the detector settings

        Parameters
        ----------
        param: Parameter
            A given parameter (within detector_settings) whose value has been changed by the user
        """
        ## TODO for your custom plugin
        if param.name() == "a_parameter_you've_added_in_self.params":
           self.controller.your_method_to_apply_this_param_change()
        else:
            pass

    def ini_stage(self, controller=None):
        """Actuator communication initialization

        Parameters
        ----------
        controller: (object)
            custom object of a PyMoDAQ plugin (Slave case). None if only one actuator by controller (Master case)

        Returns
        -------
        info: str
        initialized: bool
            False if initialization failed otherwise True
        """
        serial = Controller(self.settings['com_port'])
        self.controller = self.ini_stage_init(old_controller=controller,
                                              new_controller=Rotator(serial))
        all_info = self.controller.get('info')
        self.settings.child('serial').setValue(all_info['Serial No.'])
        self.settings.child('motor').setValue(all_info['Motor Type'])
        self.settings.child('range').setValue(all_info['Range'])
        """
        info = {'Address': addr,
                'Motor Type': int(msg[3:5], 16),
                'Serial No.': msg[5:13],
                'Year': msg[13:17],
                'Firmware': msg[17:19],
                'Thread': is_metric(msg[19]),
                'Hardware': msg[20],
                'Range': (int(msg[21:25], 16)),
                'Pulse/Rev': (int(msg[25:], 16))}
        """
        info = str(all_info)
        initialized = True
        return info, initialized

    def move_abs(self, value):
        """ Move the actuator to the absolute target defined by value

        Parameters
        ----------
        value: (float) value of the absolute target positioning
        """

        value = self.check_bound(value)  #if user checked bounds, the defined bounds are applied here
        self.target_value = value
        value = self.set_position_with_scaling(value)  # apply scaling if the user specified one

        self.controller.set_angle(value)  # when writing your own plugin replace this line

    def move_rel(self, value):
        """ Move the actuator to the relative target actuator value defined by value

        Parameters
        ----------
        value: (float) value of the relative target positioning
        """
        value = self.check_bound(self.current_position + value) - self.current_position
        self.target_value = value + self.current_position
        value = self.set_position_relative_with_scaling(value)

        self.controller.shift_angle(value)  # when writing your own plugin replace this line

    def move_home(self):
        """Call the reference method of the controller"""
        self.controller.home()  # when writing your own plugin replace this line
        self.emit_status(ThreadCommand('Update_Status', ['Some info you want to log']))

    def stop_motion(self):
        """Stop the actuator and emits move_done signal"""
        pass


if __name__ == '__main__':
    main(__file__, init=False)


